﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace SmartCowBioControl.src.objects.objects
{
    /// <summary>
    /// Classe représentant une visite.
    /// </summary>
    public class Visite
    {
        // ====================================================================================================
        // PROPRIETES
        // ====================================================================================================

        /// <summary>
        /// L'id.
        /// </summary>
        public int Id { get; private set; }

        /// <summary>
        /// L'auge.
        /// </summary>
        public int Auge { get; private set; }

        /// <summary>
        /// L'animal
        /// </summary>
        public string Animal { get; private set; }

        /// <summary>
        /// La date et heure de début.
        /// </summary>
        public DateTime DateHeureDebut { get; private set; }

        /// <summary>
        /// La date et heure de fin.
        /// </summary>
        public DateTime DateHeureFin { get; private set; }

        /// <summary>
        /// Durée de la visite.
        /// </summary>
        public TimeSpan DureeVisite => this.DateHeureFin - this.DateHeureDebut;

        // ----------------------------------------------------------------------------------------------------
        // DONNEES

        public List<Eating> Eatings { get; private set; }
        public Dictionary<DateTime, int> DonneesVideo { get; private set; }
        public List<RapidObject> RapidsSansFiltres { get; private set; }
        public List<RapidObject> RapidsAvecFiltres { get; set; }
        public Dictionary<DateTime, int> DonneesSeuil { get; set; }

        public double MinPoidsRapidSansFiltres => this.RapidsSansFiltres.Min(r => r.Poids);
        public double MaxPoidsRapidSansFiltres => this.RapidsSansFiltres.Max(r => r.Poids);
        public double MinPoidsRapidAvecFiltres => this.RapidsAvecFiltres.Min(r => r.Poids);
        public double MaxPoidsRapidAvecFiltres => this.RapidsAvecFiltres.Max(r => r.Poids);

        // ----------------------------------------------------------------------------------------------------
        // INDICES

        /// <summary>
        /// Valeur de lindice de jaccard.
        /// </summary>
        public double IndiceJaccard { get; set; }

        /// <summary>
        /// Valeur de la fréquence à 1 Vidéo.
        /// </summary>
        public double FrequenceVideo { get; set; }

        /// <summary>
        /// Valeur de la fréquence à 1 Seuil.
        /// </summary>
        public double FrequenceSeuil { get; set; }

        /// <summary>
        /// Valeur de la Pourcentage des durées à 1 Seuil.
        /// </summary>
        public double PourcentageSeuil { get; set; }

        /// <summary>
        /// Valeur de la Pourcentage des durées à 1 Video.
        /// </summary>
        public double PourcentageVideo { get; set; }

        // ----------------------------------------------------------------------------------------------------
        // LABELS

        /// <summary>
        /// Le label de la date.
        /// </summary>
        public string DateLabel => this.DateHeureDebut.ToShortDateString();

        /// <summary>
        /// Le label de l'heure de début.
        /// </summary>
        public string HeureDebutLabel => this.DateHeureDebut.ToString("HH:mm:ss");

        /// <summary>
        /// Le label de l'heure de fin.
        /// </summary>
        public string HeureFinLabel => this.DateHeureFin.ToString("HH:mm:ss");


        /// <summary>
        /// Le label de la durée de la visite.
        /// </summary>
        public string DureeVisiteLabel => this.DureeVisite.ToString(@"hh\:mm\:ss");


        /// <summary>
        /// Le label de l'indice de jaccard.
        /// </summary>
        public string IndiceJaccardLabel => this.IndiceJaccard.ToString();

        /// <summary>
        /// Le label de la fréquence à 1 pour vidéo 
        /// </summary>
        public string FrequenceVideoLabel => this.FrequenceVideo.ToString();

        /// <summary>
        /// Le label de la fréquence à 1 pour vidéo 
        /// </summary>
        public string FrequenceSeuilLabel => this.FrequenceSeuil.ToString();

        /// <summary>
        /// Le label de le pourcentage de la durée à 1 pour le seuil 
        /// </summary>
        public string PourcentageSeuilLabel => this.PourcentageSeuil.ToString();

        /// <summary>
        /// Le label de le pourcentage de la durée à 1 pour la vidéo 
        /// </summary>
        public string PourcentageVideoLabel => this.PourcentageVideo.ToString();

        // ====================================================================================================
        // CONSTRUCTEURS
        // ====================================================================================================

        /// <summary>
        /// Constructeur.
        /// </summary>
        /// <param name="id">un id</param>
        /// <param name="auge">une auge</param>
        /// <param name="animal">un animal</param>
        /// <param name="dateHeureDebut">une date et heure de début</param>
        /// <param name="dateHeureFin">une date et heure de fin</param>
        public Visite(int id, int auge, string animal, DateTime dateHeureDebut, DateTime dateHeureFin)
        {
            this.Id = id;
            this.Auge = auge;
            this.Animal = animal;
            this.DateHeureDebut = dateHeureDebut;
            this.DateHeureFin = dateHeureFin;

            this.Eatings = new List<Eating>();
            this.DonneesVideo = new Dictionary<DateTime, int>();
            this.RapidsSansFiltres = new List<RapidObject>();
            this.RapidsAvecFiltres = new List<RapidObject>();
            this.DonneesSeuil = new Dictionary<DateTime, int>();

            this.IndiceJaccard = 0;
            this.PourcentageVideo = 0;
            this.PourcentageSeuil = 0;
            this.FrequenceSeuil = 0;
            this.FrequenceVideo = 0;
        }

        // ====================================================================================================
        // METHODES PUBLIQUES
        // ====================================================================================================

        public Dictionary<DateTime, int> GetDonneesVideoPeriode(DateTime dateHeureDebut, DateTime dateHeureFin)
        {
            return this.DonneesVideo.Where(d => d.Key >= dateHeureDebut && d.Key <= dateHeureFin).ToDictionary(d => d.Key, d => d.Value);
        }

        public List<RapidObject> GetRapidsSansFiltresPeriode(DateTime dateHeureDebut, DateTime dateHeureFin)
        {
            return this.RapidsSansFiltres.Where(r => r.DateHeureDephasage >= dateHeureDebut && r.DateHeureDephasage <= dateHeureFin).ToList();
        }

        public List<RapidObject> GetRapidsAvecFiltresPeriode(DateTime dateHeureDebut, DateTime dateHeureFin)
        {
            return this.RapidsAvecFiltres.Where(r => r.DateHeureDephasage >= dateHeureDebut && r.DateHeureDephasage <= dateHeureFin).ToList();
        }

        public Dictionary<DateTime, int> GetDonneesSeuilPeriode(DateTime dateHeureDebut, DateTime dateHeureFin)
        {
            return this.DonneesSeuil.Where(d => d.Key >= dateHeureDebut && d.Key <= dateHeureFin).ToDictionary(d => d.Key, d => d.Value);
        }
    }
}
