﻿using SmartCowBioControl.src.objects;
using System;
using System.Collections.Generic;
using System.Data.SQLite;
using System.IO;
using System.Linq;

namespace SmartCowBioControl.src.database
{
    public class DatabaseService
    {
        // ====================================================================================================
        // CONSTANTES
        // ====================================================================================================

        public const string DATABASE_FILE = "database.db";
        public const string DATASOURCE = "Data Source=" + DATABASE_FILE;

        public const string TABLE_VISITE = "visite";
        public const string VISITE_ID = "id";
        public const string VISITE_ANIMAL = "animal";
        public const string VISITE_AUGE = "auge";
        public const string VISITE_DATE_DEBUT = "dateDebut";
        public const string VISITE_DATE_FIN = "dateFin";

        public const string TABLE_AUGE = "auge";
        public const string AUGE_ID = "id";
        public const string AUGE_DATE = "date";
        public const string AUGE_ANIMAL = "animal";
        public const string AUGE_POIDS = "poids";

        private static SQLiteConnection connection;
        private static SQLiteCommand command;
        private static SQLiteTransaction transaction;

        // ====================================================================================================
        // PROPRIETES
        // ====================================================================================================

        public static bool IsDatabase => File.Exists(DATABASE_FILE);

        // ====================================================================================================
        // METHODES PUBLIQUES
        // ====================================================================================================

        public static void InitDatabase()
        {
            if (IsDatabase)
            {
                File.Delete(DATABASE_FILE);
            }

            SQLiteConnection.CreateFile(DATABASE_FILE);
        }

        public static void BeginTransaction()
        {
            connection = new SQLiteConnection(DATASOURCE);
            connection.Open();

            transaction = connection.BeginTransaction();

            command = new SQLiteCommand(connection);
            command.Transaction = transaction;
        }

        public static void CloseTransaction(bool erreur)
        {
            if (erreur)
            {
                transaction.Rollback();
            }
            else
            {
                transaction.Commit();
            }

            if (connection != null) connection.Close();
        }

        public static void InsertVisites(List<BioControlVisite> visites)
        {
            command.CommandText = $"CREATE TABLE IF NOT EXISTS {TABLE_VISITE}({VISITE_ID} INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, {VISITE_ANIMAL} TEXT NOT NULL, {VISITE_AUGE} INTEGER NOT NULL, {VISITE_DATE_DEBUT} INTEGER NOT NULL, {VISITE_DATE_FIN} INTEGER NOT NULL);";
            command.ExecuteNonQuery();

            command.CommandText = $"INSERT INTO {TABLE_VISITE} ({VISITE_ANIMAL}, {VISITE_AUGE}, {VISITE_DATE_DEBUT}, {VISITE_DATE_FIN}) VALUES (@{VISITE_ANIMAL}, @{VISITE_AUGE}, @{VISITE_DATE_DEBUT}, @{VISITE_DATE_FIN})";

            foreach (BioControlVisite visite in visites)
            {
                command.Parameters.AddWithValue($"@{VISITE_ANIMAL}", visite.NumAnimal);
                command.Parameters.AddWithValue($"@{VISITE_AUGE}", visite.NumAuge);
                command.Parameters.AddWithValue($"@{VISITE_DATE_DEBUT}", visite.DateHeureDebut);
                command.Parameters.AddWithValue($"@{VISITE_DATE_FIN}", visite.DateHeureFin);
                command.ExecuteNonQuery();
            }
        }

        public static void InsertRapid(List<BioControlPoids> poids)
        {
            Dictionary<int, Dictionary<DateTime, List<BioControlPoids>>> poidsParAugesEtDates =
                poids.GroupBy(p => p.NumAuge).ToDictionary(g => g.Key, g =>
                    g.GroupBy(p => p.DateHeure.Date).ToDictionary(g2 => g2.Key, g2 => g2.ToList()));

            foreach (var poidsParAugeEtDates in poidsParAugesEtDates)
            {
                int auge = poidsParAugeEtDates.Key;

                foreach (var poidsParAugeEtDate in poidsParAugeEtDates.Value)
                {
                    DateTime date = poidsParAugeEtDate.Key;

                    command.CommandText = $"CREATE TABLE IF NOT EXISTS {TABLE_AUGE}_{auge}_{date.ToString("yyyyMMdd")} ({AUGE_ID} INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, {AUGE_DATE} INTEGER NOT NULL, {AUGE_ANIMAL} INTEGER NOT NULL, {AUGE_POIDS} INTEGER NOT NULL);";
                    command.ExecuteNonQuery();

                    command.CommandText = $"INSERT INTO {TABLE_AUGE}_{auge}_{date.ToString("yyyyMMdd")} ({AUGE_DATE}, {AUGE_ANIMAL}, {AUGE_POIDS}) VALUES (@{AUGE_DATE}, @{AUGE_ANIMAL}, @{AUGE_POIDS})";
                    foreach (BioControlPoids unPoids in poidsParAugeEtDate.Value)
                    {
                        command.Parameters.AddWithValue($"@{AUGE_DATE}", unPoids.DateHeure);
                        command.Parameters.AddWithValue($"@{AUGE_ANIMAL}", unPoids.NumAnimal);
                        command.Parameters.AddWithValue($"@{AUGE_POIDS}", unPoids.Poids);
                        command.ExecuteNonQuery();
                    }
                }
            }
        }
    }
}
