﻿using SmartCowBioControl.src.util;
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Windows.Forms;

namespace SmartCowBioControl.src.form
{
    /// <summary>
    /// Fenêtre affichant une barre de progression.
    /// </summary>
    public partial class ProgressBarForm : Form
    {
        // ====================================================================================================
        // ATTRIBUTS
        // ====================================================================================================

        /// <summary>
        /// Le traitement.
        /// </summary>
        private Func<BackgroundWorker, bool> traitement;

        /// <summary>
        /// Le nombre d'éléments du traitement.
        /// </summary>
        private int nbElementsTraitement;

        /// <summary>
        /// Le nombre d'unités du traitement.
        /// </summary>
        private long nbUnitesTraitement;

        /// <summary>
        /// Le nombre d'étapes du traitement.
        /// </summary>
        private int nbEtapesTraitement;

        // ====================================================================================================
        // PROPRIETES
        // ====================================================================================================

        /// <summary>
        /// La valeur de retour du traitement.
        /// </summary>
        public bool ValeurRetourTraitement { get; private set; }

        /// <summary>
        /// Chronomètre du traitement.
        /// </summary>
        public Stopwatch Chronometre { get; private set; }

        // ====================================================================================================
        // CONSTRUCTEURS
        // ====================================================================================================

        /// <summary>
        /// Constructeur.
        /// </summary>
        /// <param name="traitement">un traitement</param>
        /// <param name="isTraitementAnnulable">indique si le traitement est annulable</param>
        /// <param name="nbElementsTraitement">le nombre d'éléments du traitement (0 par défaut)</param>
        /// <param name="nbEtapesTraitement">le nombre d'étapes du traitement (1 par défaut)</param>
        /// <param name="titre">le titre de la fenêtre (null par défaut)</param>
        public ProgressBarForm(
            Func<BackgroundWorker, bool> traitement,
            bool isTraitementAnnulable,
            int nbElementsTraitement,
            long nbUnitesTraitement,
            int nbEtapesTraitement,
            string titre)
        {
            InitializeComponent();

            this.traitement = traitement;
            this.nbElementsTraitement = nbElementsTraitement;
            this.nbUnitesTraitement = nbUnitesTraitement;
            this.nbEtapesTraitement = nbEtapesTraitement;

            if (!String.IsNullOrWhiteSpace(titre))
            {
                this.Text = titre;
            }

            this.backgroundWorker.WorkerSupportsCancellation = isTraitementAnnulable;
            if (!isTraitementAnnulable)
            {
                Utilitaire.HideRow(this.panel, this.boutonAnnuler);
                this.Height -= 40;
            }

            if (nbElementsTraitement > 0)
            {
                this.progressBar.Maximum = nbElementsTraitement * nbEtapesTraitement;
            }
            else
            {
                Utilitaire.HideRow(this.panel, this.labelTempsEcoule);
                Utilitaire.HideRow(this.panel, this.labelElementsRestants);
                this.Height -= 60;
            }

            this.ProgressionTraitement(0, new long[] { 0, 0, 0 });

            // Sélection de la progressBar, sinon le bouton Annuler est sélectionné
            this.progressBar.Select();

            // Lancement du traitement en asynchrone
            this.backgroundWorker.RunWorkerAsync();
        }

        // ====================================================================================================
        // ACTIONS
        // ====================================================================================================

        /// <summary>
        /// Lancement du traitement.
        /// </summary>
        /// <param name="sender">le sender</param>
        /// <param name="e">l'évènement</param>
        private void backgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            this.LancementTraitement();
        }

        /// <summary>
        /// Progression du traitement.
        /// </summary>
        /// <param name="sender">le sender</param>
        /// <param name="e">l'évènement</param>
        private void backgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            this.ProgressionTraitement(e.ProgressPercentage, (long[])e.UserState);
        }

        /// <summary>
        /// Annulation du traitement.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void boutonAnnuler_Click(object sender, EventArgs e)
        {
            this.AnnulationTraitement();
        }

        /// <summary>
        /// Fin du traitement.
        /// </summary>
        /// <param name="sender">le sender</param>
        /// <param name="e">l'évènement</param>
        private void backgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.FinTraitement();
        }

        // ====================================================================================================
        // METHODES PRIVEES
        // ====================================================================================================

        /// <summary>
        /// Lancement du traitement.
        /// </summary>
        private void LancementTraitement()
        {
            if (this.traitement != null)
            {
                this.Chronometre = new Stopwatch();
                this.Chronometre.Start();
                this.ValeurRetourTraitement = this.traitement(this.backgroundWorker);
            }
        }

        /// <summary>
        /// Annulation du traitement.
        /// </summary>
        private void AnnulationTraitement()
        {
            DialogResult dialogResult = AfficheurMessage.Instance.AfficherMessageAvertissementChoix("Êtes-vous sûr de vouloir annuler le traitement ?", MessageBoxButtons.YesNo);

            if (dialogResult.Equals(DialogResult.Yes))
            {
                this.backgroundWorker.CancelAsync();

                this.boutonAnnuler.Text = "ANNULATION EN COURS...";
            }

            // Sélection de la progressBar, pour déselectionner le bouton Annuler
            this.progressBar.Select();
        }

        /// <summary>
        /// Progression du traitement.
        /// </summary>
        /// <param name="pourcentage">le pourcentage de traitement</param>
        /// <param name="parametres">les paramètres</param>
        private void ProgressionTraitement(int pourcentageTraitement, long[] parametres)
        {
            this.progressBar.Value = pourcentageTraitement;

            // Temps et éléments restants
            if (this.nbElementsTraitement > 0)
            {
                long nbElementsTraites = parametres[0];
                long nbUnitesTraitees = parametres[1];
                long nbErreurs = parametres[2];

                int nbElementsRestants = this.nbElementsTraitement - (int)nbElementsTraites;
                this.labelElementsRestantsValeur.Text = $"{nbElementsRestants} / {this.nbElementsTraitement}";
                if (nbErreurs > 0)
                {
                    this.labelElementsRestantsValeur.Text += $" ({nbErreurs} erreur{(nbErreurs > 1 ? "s" : "")})";
                }

                if (this.Chronometre != null)
                {
                    this.labelTempsEcouleValeur.Text = Utilitaire.FormatTimeSpan(this.Chronometre.Elapsed);

                    if (nbUnitesTraitees > 0)
                    {
                        long nbUnitesRestantes = this.nbUnitesTraitement - nbUnitesTraitees;
                        long tickRestant = nbUnitesRestantes * this.Chronometre.ElapsedTicks / nbUnitesTraitees;
                        this.labelTempsRestantValeur.Text = Utilitaire.FormatTimeSpan(new TimeSpan(tickRestant), true);
                    }
                }
            }
        }

        /// <summary>
        /// Fin du traitement.
        /// </summary>
        private void FinTraitement()
        {
            this.Chronometre.Stop();
            this.Close();
        }
    }
}
