﻿using SmartCowBioControl.src.util;
using System;

namespace SmartCowBioControl.src.objects
{
    public class Periode
    {
        // ====================================================================================================
        // PROPRIETES
        // ====================================================================================================

        /// <summary>
        /// Date de début.
        /// </summary>
        public DateTime DateDebut { get; private set; }

        /// <summary>
        /// Date de fin incluse.
        /// </summary>
        public DateTime DateFinIncluse { get; private set; }

        // ====================================================================================================
        // CONSTRUCTEURS
        // ====================================================================================================

        /// <summary>
        /// Constructeur.
        /// </summary>
        /// <param name="dateDebut">une date de début</param>
        /// <param name="dateFinIncluse">une date de fin incluse</param>
        public Periode(DateTime dateDebut, DateTime dateFinIncluse)
        {
            this.DateDebut = dateDebut;
            this.DateFinIncluse = dateFinIncluse;
        }

        // ====================================================================================================
        // METHODES PUBLIQUES
        // ====================================================================================================

        /// <summary>
        /// Indique si la période contient une date.
        /// </summary>
        /// <param name="date">une date</param>
        /// <param name="isDateSansHeureThis">indique s'il faut prendre la date sans l'heure pour la période courante (false par défaut)</param>
        /// <param name="isDateSansHeureArg">indique s'il faut prendre la date sans l'heure pour la période en argument (false par défaut)</param>
        /// <returns>bool</returns>
        public bool Contains(DateTime date, bool isDateSansHeureThis = false, bool isDateSansHeureArg = false)
        {
            return this.Contains(date, date, isDateSansHeureThis, isDateSansHeureArg);
        }

        /// <summary>
        /// Indique si la période contient une période donnée.
        /// </summary>
        /// <param name="dateDebut">une date de début</param>
        /// <param name="dateFin">une date de fin</param>
        /// <param name="isDateSansHeureThis">indique s'il faut prendre la date sans l'heure pour la période courante (false par défaut)</param>
        /// <param name="isDateSansHeureArg">indique s'il faut prendre la date sans l'heure pour la période en argument (false par défaut)</param>
        /// <returns>bool</returns>
        public bool Contains(DateTime dateDebut, DateTime dateFin, bool isDateSansHeureThis = false, bool isDateSansHeureArg = false)
        {
            return this.Matches(dateFin, dateDebut, true, isDateSansHeureThis, isDateSansHeureArg);
        }

        /// <summary>
        /// Indique si la période contient une période donnée.
        /// </summary>
        /// <param name="periode">une période</param>
        /// <param name="isDateSansHeureThis">indique s'il faut prendre la date sans l'heure pour la période courante (false par défaut)</param>
        /// <param name="isDateSansHeureArg">indique s'il faut prendre la date sans l'heure pour la période en argument (false par défaut)</param>
        /// <returns>bool</returns>
        public bool Contains(Periode periode, bool isDateSansHeureThis = false, bool isDateSansHeureArg = false)
        {
            return this.Contains(periode.DateDebut, periode.DateFinIncluse, isDateSansHeureThis, isDateSansHeureArg);
        }

        /// <summary>
        /// Indique si la période chevauche une période donnée.
        /// </summary>
        /// <param name="periode">une période</param>
        /// <param name="isDateSansHeureThis">indique s'il faut prendre la date sans l'heure pour la période courante (false par défaut)</param>
        /// <param name="isDateSansHeureArg">indique s'il faut prendre la date sans l'heure pour la période en argument (false par défaut)</param>
        /// <returns>bool</returns>
        public bool Matches(Periode periode, bool isDateSansHeureThis = false, bool isDateSansHeureArg = false)
        {
            return this.Matches(periode.DateDebut, periode.DateFinIncluse, true, isDateSansHeureThis, isDateSansHeureArg);
        }

        /// <summary>
        /// Indique si la période chevauche une période donnée.
        /// </summary>
        /// <param name="dateDebut">une date de début</param>
        /// <param name="dateFin">une date de fin</param>
        /// <param name="isDateFinIncluse">indique si la date de fin est incluse (false par défaut)</param>
        /// <param name="isDateSansHeureThis">indique s'il faut prendre la date sans l'heure pour la période courante (false par défaut)</param>
        /// <param name="isDateSansHeureArg">indique s'il faut prendre la date sans l'heure pour la période en argument (false par défaut)</param>
        /// <returns>bool</returns>
        public bool Matches(
            DateTime dateDebut,
            DateTime dateFin,
            bool isDateFinIncluse = false,
            bool isDateSansHeureThis = false,
            bool isDateSansHeureArg = false)
        {
            DateTime dateDebutThis = isDateSansHeureThis ? this.DateDebut.Date : this.DateDebut;
            DateTime dateFinThis = isDateSansHeureThis ? this.DateFinIncluse.Date : this.DateFinIncluse;

            DateTime dateDebutArg = isDateSansHeureArg ? dateDebut.Date : dateDebut;
            DateTime dateFinArg = isDateSansHeureArg ? dateFin.Date : dateFin;

            return (isDateFinIncluse ? dateDebutThis <= dateFinArg : dateDebutThis < dateFinArg) && (Utilitaire.IsDateNull(dateFinThis) || dateFinThis >= dateDebutArg);
        }

        /// <summary>
        /// Retourne une période de chevauchement.
        /// </summary>
        /// <param name="periode">une période</param>
        /// <returns>la période de chevauchement</returns>
        public Periode GetPeriodeChevauchement(Periode periode)
        {
            return this.Matches(periode) ?
                new Periode(
                    Utilitaire.Max(this.DateDebut, periode.DateDebut),
                    Utilitaire.Min(this.DateFinIncluse, periode.DateFinIncluse))
                : null;
        }

        /// <summary>
        /// Retourne une période de chevauchement.
        /// </summary>
        /// <param name="dateHeureDebut">une date et heure de début</param>
        /// <param name="dateHeureFinIncluse">une date et heure de fin incluse</param>
        /// <returns>la période de chevauchement</returns>
        public Periode GetPeriodeChevauchement(DateTime dateHeureDebut, DateTime dateHeureFinIncluse)
        {
            return this.GetPeriodeChevauchement(new Periode(dateHeureDebut, dateHeureFinIncluse));
        }
    }
}
