﻿using SmartCowBioControl.src.util;
using System;
using System.Collections.Generic;
using System.Linq;

namespace SmartCowBioControl.src.objects.objects
{
    /// <summary>
    /// Classe représentant une visite.
    /// </summary>
    public class Visite
    {
        // ====================================================================================================
        // PROPRIETES
        // ====================================================================================================

        /// <summary>
        /// L'id.
        /// </summary>
        public int Id { get; private set; }

        /// <summary>
        /// L'auge.
        /// </summary>
        public int Auge { get; private set; }

        /// <summary>
        /// L'animal
        /// </summary>
        public string Animal { get; private set; }

        /// <summary>
        /// La date et heure de début.
        /// </summary>
        public DateTime DateHeureDebut { get; private set; }

        /// <summary>
        /// La date et heure de fin.
        /// </summary>
        public DateTime DateHeureFin { get; private set; }

        /// <summary>
        /// Durée de la visite.
        /// </summary>
        public TimeSpan DureeVisite => this.DateHeureFin - this.DateHeureDebut;

        /// <summary>
        /// Durée de la visite en secondes.
        /// </summary>
        public long DureeVisiteSecondes => (long)this.DureeVisite.TotalSeconds;

        // ----------------------------------------------------------------------------------------------------
        // DONNEES

        public List<RapidObject> RapidsSansFiltres { get; private set; }
        public List<RapidObject> RapidsAvecFiltres { get; set; }
        public Dictionary<DateTime, int> DonneesSeuil { get; set; }

        public List<RapidObject> MaxLocaux { get; set; }
        public double Seuil { get; set; }

        public double MinPoidsRapidSansFiltres => this.RapidsSansFiltres.Min(r => r.Poids);
        public double MaxPoidsRapidSansFiltres => this.RapidsSansFiltres.Max(r => r.Poids);
        public double MinPoidsRapidAvecFiltres => this.RapidsAvecFiltres.Min(r => r.Poids);
        public double MaxPoidsRapidAvecFiltres => this.RapidsAvecFiltres.Max(r => r.Poids);

        // ----------------------------------------------------------------------------------------------------
        // INDICES

        public int nbEtapesTraiteesCalculIndices { get; set; }

        public int NbMaxLocaux => this.MaxLocaux == null ? 0 : this.MaxLocaux.Count;

        /// <summary>
        /// Valeur de la fréquence à 1 Seuil.
        /// </summary>
        public double FrequenceSeuil { get; set; }

        /// <summary>
        /// Valeur de la Pourcentage des durées à 1 Seuil.
        /// </summary>
        public double PourcentageSeuil { get; set; }

        // ----------------------------------------------------------------------------------------------------
        // LABELS

        /// <summary>
        /// Le label de la date.
        /// </summary>
        public string DateLabel => this.DateHeureDebut.ToShortDateString();

        /// <summary>
        /// Le label de l'heure de début.
        /// </summary>
        public string HeureDebutLabel => this.DateHeureDebut.ToString("HH:mm:ss");

        /// <summary>
        /// Le label de l'heure de fin.
        /// </summary>
        public string HeureFinLabel => this.DateHeureFin.ToString("HH:mm:ss");


        /// <summary>
        /// Le label de la durée de la visite.
        /// </summary>
        public string DureeVisiteLabel => this.DureeVisite.ToString(@"hh\:mm\:ss");

        /// <summary>
        /// Le label de la fréquence à 1 pour vidéo 
        /// </summary>
        public string FrequenceSeuilLabel => this.FrequenceSeuil.ToString();

        /// <summary>
        /// Le label de le pourcentage de la durée à 1 pour le seuil 
        /// </summary>
        public string PourcentageSeuilLabel => this.PourcentageSeuil.ToString();

        // ====================================================================================================
        // CONSTRUCTEURS
        // ====================================================================================================

        /// <summary>
        /// Constructeur.
        /// </summary>
        /// <param name="id">un id</param>
        /// <param name="animal">un animal</param>
        /// <param name="auge">une auge</param>
        /// <param name="dateHeureDebut">une date et heure de début</param>
        /// <param name="dateHeureFin">une date et heure de fin</param>
        public Visite(int id, string animal, int auge, DateTime dateHeureDebut, DateTime dateHeureFin)
        {
            this.Id = id;
            this.Auge = auge;
            this.Animal = animal;
            this.DateHeureDebut = dateHeureDebut;
            this.DateHeureFin = dateHeureFin;

            this.RapidsSansFiltres = new List<RapidObject>();
            this.RapidsAvecFiltres = new List<RapidObject>();
            this.DonneesSeuil = new Dictionary<DateTime, int>();
            this.MaxLocaux = new List<RapidObject>();

            this.PourcentageSeuil = 0;
            this.FrequenceSeuil = 0;
        }

        // ====================================================================================================
        // METHODES PUBLIQUES
        // ====================================================================================================

        public List<RapidObject> GetRapidsSansFiltresPeriode(DateTime dateHeureDebut, DateTime dateHeureFin)
        {
            return this.RapidsSansFiltres.Where(r => r.DateHeure >= dateHeureDebut && r.DateHeure <= dateHeureFin).ToList();
        }

        public List<RapidObject> GetRapidsAvecFiltresPeriode(DateTime dateHeureDebut, DateTime dateHeureFin)
        {
            return this.RapidsAvecFiltres.Where(r => r.DateHeure >= dateHeureDebut && r.DateHeure <= dateHeureFin).ToList();
        }

        public Dictionary<DateTime, int> GetDonneesSeuilPeriode(DateTime dateHeureDebut, DateTime dateHeureFin)
        {
            return this.DonneesSeuil.Where(d => d.Key >= dateHeureDebut && d.Key <= dateHeureFin).ToDictionary(d => d.Key, d => d.Value);
        }

        public List<RapidObject> GetMaxLocauxPeriode(DateTime dateHeureDebut, DateTime dateHeureFin)
        {
            return this.MaxLocaux.Where(r => r.DateHeure >= dateHeureDebut && r.DateHeure <= dateHeureFin).ToList();
        }

        public bool AjouterEtapeCalculIndices(ReportProgressParameters reportProgressParameters)
        {
            this.nbEtapesTraiteesCalculIndices++;
            if (reportProgressParameters != null) reportProgressParameters.AddPercentage();

            return
                reportProgressParameters == null ||
                reportProgressParameters.BackgroundWorker == null ||
                !reportProgressParameters.BackgroundWorker.CancellationPending;
        }
    }
}
