﻿using ClosedXML.Excel;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;

namespace SmartCowBioControl.src.objects.fichiers
{
    /// <summary>
    /// Classe représentant un fichier Rapid.
    /// </summary>
    public class FichierVisites
    {
        // ====================================================================================================
        // CONSTANTES
        // ====================================================================================================

        public const string FORMAT_FICHIER = "visites ???????? ????????.xlsx";
        private const string FORMAT_DATE = "yyyyMMdd";
        private const int INDICE_DATE_DEBUT = 8;
        private const int INDICE_DATE_FIN = 17;

        private const string COLONNE_ANIMAL = "ANIMAL";
        private const string COLONNE_AUGE = "AUGE";
        private const string COLONNE_ALIMENT = "ALIMENT";
        private const string COLONNE_DEBUT = "DEBUT";
        private const string COLONNE_FIN = "FIN";
        private const string COLONNE_QUANTITE = "QUANTITE";


        // ====================================================================================================
        // METHODES PUBLIQUES
        // ====================================================================================================

        /// <summary>
        /// Retourne les visites d'une période.
        /// </summary>
        /// <param name="dossier">un dossier</param>
        /// <param name="dateDebut">une date de début</param>
        /// <param name="dateFinIncluse">une date de fin incluse</param>
        /// <returns>la liste des visites</returns>
        public static List<BioControlVisite> GetVisites(string dossier, DateTime dateDebut, DateTime dateFinIncluse)
        {
            List<BioControlVisite> visites = new List<BioControlVisite>();

            string[] cheminsFichiers = Directory.GetFiles(dossier, "visites ???????? ????????.xlsx");

            if (cheminsFichiers.Length > 0)
            {
                Periode periode = new Periode(dateDebut, dateFinIncluse);
                Dictionary<Periode, List<string>> cheminsFichiersParPeriodes = cheminsFichiers.GroupBy(c => GetPeriode(c), c => c).ToDictionary(g => g.Key, g => g.ToList());

                foreach (KeyValuePair<Periode, List<string>> kvp in cheminsFichiersParPeriodes.Where(kvp => kvp.Key.Matches(periode)))
                {
                    foreach (string cheminFichier in kvp.Value)
                    {
                        visites.AddRange(GetVisites(cheminFichier).Where(v => periode.Contains(v.DateHeureFin)));
                    }
                }
            }

            return visites;
        }

        // ====================================================================================================
        // METHODES PRIVEES
        // ====================================================================================================

        /// <summary>
        /// Retourne une période à partir d'un chemin de fichier.
        /// </summary>
        /// <param name="cheminFichier">un chemin de fichier</param>
        /// <returns>une période</returns>
        private static Periode GetPeriode(string cheminFichier)
        {
            DateTime dateDebut;
            DateTime dateFin;

            string nomFichier = Path.GetFileName(cheminFichier);

            int tailleDate = FORMAT_DATE.Length;

            string dateDebutStr = nomFichier.Substring(INDICE_DATE_DEBUT, tailleDate);
            dateDebut = DateTime.ParseExact(dateDebutStr, FORMAT_DATE, CultureInfo.CurrentCulture);

            string dateFinStr = nomFichier.Substring(INDICE_DATE_FIN, tailleDate);
            dateFin = DateTime.ParseExact(dateFinStr, FORMAT_DATE, CultureInfo.CurrentCulture);

            // La date de fin est renseignée à 23h59.
            dateFin = dateFin.AddDays(1).AddSeconds(-1);

            return new Periode(dateDebut, dateFin);
        }

        /// <summary>
        /// Retourne les visites d'un fichier.
        /// </summary>
        /// <param name="cheminFichier">un chemin de fichier</param>
        /// <returns>les visites</returns>
        private static List<BioControlVisite> GetVisites(string cheminFichier)
        {
            List<BioControlVisite> visites = new List<BioControlVisite>();

            XLWorkbook workbook = new XLWorkbook(cheminFichier);
            IXLWorksheet worksheet;
            if (workbook.TryGetWorksheet("Visites", out worksheet))
            {
                int indiceColonneAnimal = -1;
                int indiceColonneAuge = -1;
                int indiceColonneAliment = -1;
                int indiceColonneDebut = -1;
                int indiceColonneFin = -1;
                int indiceColonneQuantite = -1;

                int ligne = 1;

                foreach (IXLCell cell in worksheet.Row(ligne).CellsUsed())
                {
                    string valeur = cell.GetString();
                    int indiceColonne = cell.WorksheetColumn().ColumnNumber();

                    switch (valeur)
                    {
                        case COLONNE_ANIMAL:
                            indiceColonneAnimal = indiceColonne;
                            break;

                        case COLONNE_AUGE:
                            indiceColonneAuge = indiceColonne;
                            break;

                        case COLONNE_ALIMENT:
                            indiceColonneAliment = indiceColonne;
                            break;

                        case COLONNE_DEBUT:
                            indiceColonneDebut = indiceColonne;
                            break;

                        case COLONNE_FIN:
                            indiceColonneFin = indiceColonne;
                            break;

                        case COLONNE_QUANTITE:
                            indiceColonneQuantite = indiceColonne;
                            break;
                    }
                }

                if (indiceColonneAnimal != -1 &&
                    indiceColonneAuge != -1 &&
                    indiceColonneAliment != -1 &&
                    indiceColonneDebut != -1 &&
                    indiceColonneFin != -1 &&
                    indiceColonneQuantite != -1)
                {
                    foreach (IXLRow row in worksheet.RowsUsed().Skip(1))
                    {
                        int animal = row.Cell(indiceColonneAnimal).GetValue<int>();
                        int auge = row.Cell(indiceColonneAuge).GetValue<int>();
                        string aliment = row.Cell(indiceColonneAliment).GetString();
                        DateTime dateDebut = row.Cell(indiceColonneDebut).GetDateTime();
                        DateTime dateFin = row.Cell(indiceColonneFin).GetDateTime();
                        double quantite = row.Cell(indiceColonneQuantite).GetDouble();

                        visites.Add(new BioControlVisite(animal, auge, aliment, dateDebut, dateFin, (int)(quantite * 1000)));
                    }
                }
            }

            return visites;
        }
    }
}
