﻿using SmartCowBioControl.src.calcul;
using SmartCowBioControl.src.database;
using SmartCowBioControl.src.objects;
using SmartCowBioControl.src.objects.objects;
using SmartCowBioControl.src.util;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data.SQLite;
using System.Linq;

namespace SmartCowBioControl.src.services
{
    /// <summary>
    /// Classe de service pour les visites.
    /// </summary>
    public static class VisitesService
    {
        // ====================================================================================================
        // ATTRIBUTS
        // ====================================================================================================

        private static Dictionary<Visite, Dictionary<int, List<RapidObject>>> rapidReechantillonnes = new Dictionary<Visite, Dictionary<int, List<RapidObject>>>();

        // ====================================================================================================
        // METHODES PUBLIQUES
        // ====================================================================================================

        /// <summary>
        /// Retourne les visites.
        /// </summary>
        /// <returns>les visites</returns>
        public static List<Visite> GetVisites()
        {
            List<Visite> visites = new List<Visite>();

            if (DatabaseService.IsDatabase)
            {
                SQLiteConnection connection = null;
                SQLiteDataReader dr = null;

                try
                {
                    connection = new SQLiteConnection(DatabaseService.DATASOURCE);
                    connection.Open();

                    SQLiteCommand command = new SQLiteCommand(connection);
                    command.CommandText =
                        $"SELECT {DatabaseService.VISITE_ID}, {DatabaseService.VISITE_ANIMAL}, {DatabaseService.VISITE_AUGE}, {DatabaseService.VISITE_DATE_DEBUT}, {DatabaseService.VISITE_DATE_FIN} FROM {DatabaseService.TABLE_VISITE}";

                    dr = command.ExecuteReader();

                    while (dr.Read())
                    {
                        int id = dr.GetInt32(0);
                        string animal = dr.GetString(1);
                        int auge = dr.GetInt32(2);
                        DateTime dateHeureDebut = dr.GetDateTime(3);
                        DateTime dateHeureFin = dr.GetDateTime(4);

                        Visite visite = new Visite(
                            id,
                            animal,
                            auge,
                            dateHeureDebut,
                            dateHeureFin);

                        // enlever les visites minuscules sans eating (cause crash)
                        if (visite.DateHeureFin > dr.GetDateTime(3).AddSeconds(20))
                        {
                            visites.Add(visite);
                        }
                    }
                }
                catch (Exception) { }
                finally
                {
                    if (dr != null) dr.Close();
                    if (connection != null) connection.Close();
                }
            }

            return visites;
        }

        /// <summary>
        /// Récupérer les données d'une visite.
        /// </summary>
        /// <param name="visite">une visite</param>
        /// <param name="parametres">des paramètres</param>
        public static bool RecupererDonnees(Visite visite, Parametres parametres, ReportProgressParameters reportProgressParameters = null)
        {
            RecupererDonneesRapid(visite);
            visite.RapidsAvecFiltres = GetCopie(visite.RapidsSansFiltres);
            if (!visite.AjouterEtapeCalculIndices(reportProgressParameters))
            {
                return false;
            }

            if (parametres.IsPasDeTempsActif)
            {
                visite.RapidsAvecFiltres = GetCopie(GetRapidReechantillonnes(visite, parametres.PasDeTemps, reportProgressParameters == null ? null : reportProgressParameters.BackgroundWorker));
            }
            if (!visite.AjouterEtapeCalculIndices(reportProgressParameters))
            {
                return false;
            }

            if (parametres.IsFiltreHauteFrequenceActif)
            {
                CalculFiltres.FiltreHauteFrequence(visite.RapidsAvecFiltres, parametres.FiltreHauteFrequence, parametres.PasDeTemps);
            }
            if (!visite.AjouterEtapeCalculIndices(reportProgressParameters))
            {
                return false;
            }

            if (parametres.IsFiltreBasseFrequenceActif)
            {
                CalculFiltres.FiltreBasseFrequence(visite.RapidsAvecFiltres, parametres.FiltreBasseFrequence, parametres.PasDeTemps);
            }
            if (!visite.AjouterEtapeCalculIndices(reportProgressParameters))
            {
                return false;
            }

            if (parametres.IsNormalisationActif)
            {
                CalculFiltres.Normalisation(visite.RapidsAvecFiltres);
            }
            if (!visite.AjouterEtapeCalculIndices(reportProgressParameters))
            {
                return false;
            }

            if (parametres.IsSeuilActif)
            {
                List<RapidObject> maxLocaux;
                double seuil;
                visite.DonneesSeuil = CalculFiltres.Seuil(
                    visite.RapidsAvecFiltres,
                    parametres.FenetreCalculMaxLocaux,
                    parametres.SeuilNbDecimales,
                    parametres.SeuilMin,
                    parametres.SeuilMax,
                    out maxLocaux,
                    out seuil);
                visite.MaxLocaux = maxLocaux;
                visite.Seuil = seuil;
            }

            return visite.AjouterEtapeCalculIndices(reportProgressParameters);
        }

        // ====================================================================================================
        // METHODES PRIVEES
        // ====================================================================================================

        /// <summary>
        /// Récupère les données Rapid d'une visite.
        /// </summary>
        /// <param name="visite">une visite</param>
        private static void RecupererDonneesRapid(Visite visite)
        {
            if (visite.RapidsSansFiltres.Count == 0)
            {
                string requete = GetSelectRapid(visite.Auge, visite.DateHeureDebut.Date, visite.DateHeureDebut, visite.DateHeureFin);
                if (visite.DateHeureDebut.Date != visite.DateHeureFin.Date)
                {
                    requete += " UNION " + GetSelectRapid(visite.Auge, visite.DateHeureFin.Date, visite.DateHeureDebut, visite.DateHeureFin);
                }
                requete += $" ORDER BY {DatabaseService.AUGE_DATE}";

                SQLiteConnection connection = new SQLiteConnection(DatabaseService.DATASOURCE);
                connection.Open();

                SQLiteCommand command = new SQLiteCommand(connection);
                command.CommandText = requete;

                SQLiteDataReader dr = command.ExecuteReader();

                DateTime dateHeurePrecedente = DateTime.MinValue;
                while (dr.Read())
                {
                    DateTime dateHeure = dr.GetDateTime(0);
                    int animal = dr.GetInt32(1);
                    int poids = dr.GetInt32(2);

                    // Le test est valide car on fait un order by dans la requête
                    if (dateHeure != dateHeurePrecedente)
                    {
                        visite.RapidsSansFiltres.Add(new RapidObject(animal, dateHeure, poids));
                        dateHeurePrecedente = dateHeure;
                    }
                }

                dr.Close();
                connection.Close();
            }
        }

        private static string GetSelectRapid(int auge, DateTime jour, DateTime dateHeureDebut, DateTime dateHeureFin)
        {
            return
                $"SELECT {DatabaseService.AUGE_DATE}, {DatabaseService.AUGE_ANIMAL}, {DatabaseService.AUGE_POIDS}" +
                $" FROM {DatabaseService.TABLE_AUGE}_{auge}_{jour.ToString("yyyyMMdd")}" +
                $" WHERE {DatabaseService.AUGE_DATE} BETWEEN '{dateHeureDebut.ToString("yyyy-MM-dd HH:mm:ss.f")}'" +
                $" AND '{dateHeureFin.ToString("yyyy-MM-dd HH:mm:ss.f")}'";
        }

        private static List<RapidObject> GetRapidReechantillonnes(Visite visite, int pasDeTemps, BackgroundWorker backgroundWorker = null)
        {
            if (!rapidReechantillonnes.ContainsKey(visite))
            {
                rapidReechantillonnes.Add(visite, new Dictionary<int, List<RapidObject>>());
            }

            if (!rapidReechantillonnes[visite].ContainsKey(pasDeTemps))
            {
                rapidReechantillonnes[visite].Add(pasDeTemps, Reechantillonnage.GetRapidReechantillonnes(visite.RapidsSansFiltres, pasDeTemps, visite.DateHeureDebut, visite.DateHeureFin, backgroundWorker));
            }

            return rapidReechantillonnes[visite][pasDeTemps];
        }

        private static List<RapidObject> GetCopie(List<RapidObject> rapids)
        {
            return rapids.Select(r => r.Clone).ToList();
        }
    }
}
