﻿using SmartCowBioControl.src.form;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace SmartCowBioControl.src.util
{
    /// <summary>
    /// Classe utilitaire.
    /// </summary>
    public static class Utilitaire
    {
        // ====================================================================================================
        // CONSTANTES
        // ====================================================================================================

        // --------------------------------------------------------------------------------
        // Constantes pour les méthodes GetHashCode.
        // cf. http://www.loganfranken.com/blog/692/overriding-equals-in-c-part-2/

        /// <summary>
        /// Constante de base pour le hachage.
        /// </summary>
        public static int HASHING_BASE { get { unchecked { return (int)2166136261; } } }

        /// <summary>
        /// Constante de multiplication pour le hachage.
        /// </summary>
        public const int HASHING_MULTIPLIER = 16777619;

        // --------------------------------------------------------------------------------

        // ====================================================================================================
        // PROPRIETES
        // ====================================================================================================

        /// <summary>
        /// Séparateur décimal.
        /// </summary>
        public static char SeparateurDecimal => CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator[0];

        // ====================================================================================================
        // METHODES PUBLIQUES
        // ====================================================================================================

        /// <summary>
        /// Retourne une date formatée.
        /// </summary>
        /// <param name="date">un objet date</param>
        /// <returns>la date formatée</returns>
        public static string AFFICHE_DATE(DateTime date)
        {
            String retour = "";
            if (!date.Equals(new DateTime(1, 1, 1)))
            {
                retour = date.ToShortDateString();
            }
            return retour;
        }

        /// <summary>
        /// Indique si une date est nulle.
        /// </summary>
        /// <param name="date">une date</param>
        /// <returns>bool</returns>
        public static bool IsDateNull(DateTime date)
        {
            return date.Equals(new DateTime());
        }

        /// <summary>
        /// Retourne une date formatée, ou une chaîne indiquant que la date est nulle.
        /// </summary>
        /// <param name="date">une date</param>
        /// <param name="chaineSiNulle">une chaîne à afficher si la date est nulle ("" par défaut)</param>
        /// <returns>la date formatée</returns>
        public static string FormatShortDateNullable(DateTime date, string chaineSiNulle = "")
        {
            return IsDateNull(date) ? chaineSiNulle : date.ToShortDateString();
        }

        /// <summary>
        /// Retourne une date au format ISO.
        /// </summary>
        /// <param name="date">une date</param>
        /// <returns>la date formatée</returns>
        public static string FormatDateISO(DateTime date)
        {
            return date.ToString("yyyy-MM-dd");
        }

        /// <summary>
        /// Retourne une date au format jj/mm.
        /// </summary>
        /// <param name="date">une date</param>
        /// <returns>la date formatée</returns>
        public static string FormatDateJourMois(DateTime date)
        {
            return date.ToString("dd/MM");
        }

        /// <summary>
        /// Formate un TimeSpan.
        /// </summary>
        /// <param name="timeSpan">le TimeSpan</param>
        /// <returns>le TimeSpan formaté</returns>
        public static string FormatTimeSpan(TimeSpan timeSpan, bool mini = false)
        {
            IList<string> toString = new List<string>();

            int heures = (int)timeSpan.TotalHours;
            int minutes = heures == 0 ? (int)timeSpan.TotalMinutes : timeSpan.Minutes;
            int secondes = timeSpan.Seconds;

            if (heures > 0)
            {
                toString.Add($"{heures:d}h");
            }

            if (minutes > 0 && (!mini || heures < 2))
            {
                toString.Add($"{minutes:d}mn");
            }

            if (!mini && (secondes > 0 || heures == 0 && minutes == 0))
            {
                toString.Add($"{secondes:d}s");
            }

            return string.Join(" ", toString);
        }

        /// <summary>
        /// Retourne le nom de jour.
        /// </summary>
        /// <param name="date">une date</param>
        /// <returns>le nom de jour</returns>
        public static string GetNomJour(DateTime date)
        {
            return date.ToString("dddd", new CultureInfo("fr-FR"));
        }

        /// <summary>
        /// Retourne l'indice de semaine d'un jour (0 => lundi ... 6 => dimanche).
        /// </summary>
        /// <param name="jour">un jour</param>
        /// <returns>l'indice</returns>
        public static int GetIndiceJour(DateTime jour)
        {
            int indiceJour = jour.DayOfWeek - DayOfWeek.Monday;
            return (indiceJour < 0) ? 6 : indiceJour;
        }

        /// <summary>
        /// Retourne le lundi de la semaine d'un jour donné.
        /// </summary>
        /// <param name="jour">un jour</param>
        /// <returns>le lundi</returns>
        public static DateTime GetLundi(DateTime jour)
        {
            return jour.AddDays(-GetIndiceJour(jour));
        }

        // ----------------------------------------------------------------------------------------------------
        // DATES : MAX

        /// <summary>
        /// Retourne le max d'une liste de dates.
        /// </summary>
        /// <param name="dates">des dates</param>
        /// <returns>la date max</returns>
        public static DateTime Max(params DateTime[] dates)
        {
            DateTime max = DateTime.MinValue;

            foreach (DateTime date in dates)
            {
                max = date > max ? date : max;
            }

            return max;
        }

        /// <summary>
        /// Retourne le max d'une liste de dates.
        /// </summary>
        /// <param name="dates">des dates</param>
        /// <returns>la date max</returns>
        public static DateTime Max(IList<DateTime> dates)
        {
            return Max(dates.ToArray());
        }

        /// <summary>
        /// Retourne le max d'une liste de dates.
        /// </summary>
        /// <param name="dates">des dates</param>
        /// <returns>la date max</returns>
        public static DateTime? Max(params DateTime?[] dates)
        {
            DateTime? max = null;

            foreach (DateTime? date in dates)
            {
                max = !max.HasValue || date.HasValue && date > max ? date : max;
            }

            return max;
        }

        /// <summary>
        /// Retourne le max d'une liste de dates.
        /// </summary>
        /// <param name="dates">des dates</param>
        /// <returns>la date max</returns>
        public static DateTime? Max(IList<DateTime?> dates)
        {
            return Max(dates.ToArray());
        }

        // ----------------------------------------------------------------------------------------------------
        // DATES : MIN

        /// <summary>
        /// Retourne le min d'une liste de dates.
        /// </summary>
        /// <param name="dates">des dates</param>
        /// <returns>la date min</returns>
        public static DateTime Min(params DateTime[] dates)
        {
            DateTime min = DateTime.MaxValue;

            foreach (DateTime date in dates)
            {
                min = date < min ? date : min;
            }

            return min;
        }

        /// <summary>
        /// Retourne le min d'une liste de dates.
        /// </summary>
        /// <param name="dates">des dates</param>
        /// <returns>la date min</returns>
        public static DateTime Min(IList<DateTime> dates)
        {
            return Min(dates.ToArray());
        }

        /// <summary>
        /// Retourne le min d'une liste de dates.
        /// </summary>
        /// <param name="dates">des dates</param>
        /// <returns>la date min</returns>
        public static DateTime? Min(params DateTime?[] dates)
        {
            DateTime? min = null;

            foreach (DateTime? date in dates)
            {
                min = !min.HasValue || date.HasValue && date < min ? date : min;
            }

            return min;
        }

        /// <summary>
        /// Retourne le min d'une liste de dates.
        /// </summary>
        /// <param name="dates">des dates</param>
        /// <returns>la date min</returns>
        public static DateTime? Min(IList<DateTime?> dates)
        {
            return Min(dates.ToArray());
        }

        // ----------------------------------------------------------------------------------------------------

        /// <summary>
        /// Retourne un entier à partir d'une chaîne.
        /// </summary>
        /// <param name="s">une chaîne</param>
        /// <returns>l'entier</returns>
        public static int? ToNullableInt(String s)
        {
            int i;
            if (int.TryParse(s, out i)) return i;
            return null;
        }

        /// <summary>
        /// Retourne la valeur de propriété d'un objet.
        /// </summary>
        /// <param name="objet">un objet</param>
        /// <param name="propriete">une propriété</param>
        /// <returns>la valeur de propriété</returns>
        public static object GetValue(object objet, string propriete)
        {
            return objet.GetType().GetProperty(propriete).GetValue(objet, null);
        }

        /// <summary>
        /// Fusionne une liste d'anciens et de nouveaux objets.
        /// </summary>
        /// <typeparam name="T">un type d'objets</typeparam>
        /// <param name="anciensObjets">une liste d'anciens objets</param>
        /// <param name="nouveauxObjets">une liste de nouveaux objets</param>
        public static void Merge<T>(ISet<T> anciensObjets, IEnumerable<T> nouveauxObjets)
        {
            // Objets supprimés
            foreach (T objetSupprime in anciensObjets.Except(nouveauxObjets).ToList())
            {
                anciensObjets.Remove(objetSupprime);
            }

            // Objets ajoutés
            foreach (T nouvelObjet in nouveauxObjets.Except(anciensObjets))
            {
                anciensObjets.Add(nouvelObjet);
            }
        }

        /// <summary>
        /// Indique si 2 chaînes sont égales ou si elles sont toutes deux vides.
        /// </summary>
        /// <param name="str1">une chaîne 1</param>
        /// <param name="str2">une chaîne 2</param>
        /// <returns><code>true</code> si les chaînes sont égales, <code>false</code> sinon</returns>
        public static bool StringEqualsOrBlank(string str1, string str2)
        {
            return String.IsNullOrWhiteSpace(str1) && String.IsNullOrWhiteSpace(str2) || str1 == str2;
        }

        /// <summary>
        /// Retourne une chaîne avec le séparateur décimal correct.
        /// </summary>
        /// <param name="s">une chaîne</param>
        /// <returns></returns>
        public static string GetStringSeparateurDecimal(string s)
        {
            char separateurDecimal = SeparateurDecimal;
            return s == null ? "" : s.Replace('.', separateurDecimal).Replace(',', separateurDecimal);
        }

        /// <summary>
        /// Transforme une chaîne en int et indique si la transformation est correcte.
        /// </summary>
        /// <param name="chaine">une chaîne</param>
        /// <param name="valeur">une valeur (sortie)</param>
        /// <param name="videToZero">indique si une valeur vide doit être considérée comme zéro (false par défaut)</param>
        /// <returns><code>true</code> si la transformation est correcte, <code>false</code> sinon</returns>
        public static bool TryGetInt(string chaine, out int valeur, bool videToZero = false)
        {
            if (videToZero && String.IsNullOrWhiteSpace(chaine))
            {
                chaine = "0";
            }

            return int.TryParse(chaine, out valeur);
        }

        /// <summary>
        /// Indique si une valeur est un int.
        /// </summary>
        /// <param name="chaine">une chaîne</param>
        /// <param name="videToZero">indique si une valeur vide doit être considérée comme zéro (false par défaut)</param>
        /// <returns><code>true</code> si la valeur est un int, <code>false</code> sinon</returns>
        public static bool IsInt(string chaine, bool videToZero = false)
        {
            int valeur;
            return TryGetInt(chaine, out valeur, videToZero);
        }

        /// <summary>
        /// Transforme une chaîne en decimal et indique si la transformation est correcte.
        /// </summary>
        /// <param name="chaine">une chaîne</param>
        /// <param name="valeur">une valeur (sortie)</param>
        /// <param name="videToZero">indique si une valeur vide doit être considérée comme zéro (false par défaut)</param>
        /// <returns><code>true</code> si la transformation est correcte, <code>false</code> sinon</returns>
        public static bool TryGetDecimal(string chaine, out decimal valeur, bool videToZero = false)
        {
            string chaineSeparateurDecimal = GetStringSeparateurDecimal(chaine);

            if (videToZero && String.IsNullOrWhiteSpace(chaineSeparateurDecimal))
            {
                chaineSeparateurDecimal = "0";
            }

            return decimal.TryParse(chaineSeparateurDecimal, out valeur);
        }

        /// <summary>
        /// Indique si une valeur est décimale.
        /// </summary>
        /// <param name="chaine">une chaîne</param>
        /// <param name="videToZero">indique si une valeur vide doit être considérée comme zéro (false par défaut)</param>
        /// <returns><code>true</code> si la valeur est décimale, <code>false</code> sinon</returns>
        public static bool IsDecimal(string chaine, bool videToZero = false)
        {
            decimal valeur;
            return TryGetDecimal(chaine, out valeur, videToZero);
        }

        /// <summary>
        /// Retourne une chaîne en majuscule.
        /// </summary>
        /// <param name="s">une chaîne</param>
        /// <returns>la chaîne en majuscule</returns>
        public static string ToUpper(string s)
        {
            return s == null ? "" : s.ToUpper();
        }

        /// <summary>
        /// Indique si un nombre est pair.
        /// </summary>
        /// <param name="nombre">un nombre</param>
        /// <returns>bool</returns>
        public static bool IsPair(int nombre)
        {
            return nombre % 2 == 0;
        }

        public static String RemoveDiacritics(String s)
        {
            String normalizedString = s.Normalize(NormalizationForm.FormD);
            StringBuilder stringBuilder = new StringBuilder();

            for (int i = 0; i < normalizedString.Length; i++)
            {
                Char c = normalizedString[i];
                if (System.Globalization.CharUnicodeInfo.GetUnicodeCategory(c) != System.Globalization.UnicodeCategory.NonSpacingMark)
                    stringBuilder.Append(c);
            }

            return stringBuilder.ToString();
        }

        /// <summary>
        /// Sépare une chaîne de caractères.
        /// </summary>
        /// <param name="chaine">une chaîne</param>
        /// <param name="separateur">un séparateur</param>
        /// <returns>le tableau de chaînes</returns>
        public static string[] Split(string chaine, string separateur)
        {
            return chaine.Split(new string[] { separateur }, StringSplitOptions.RemoveEmptyEntries);
        }

        /// <summary>
        /// Sépare une chaîne de caractères par ses espaces.
        /// </summary>
        /// <param name="chaine">une chaîne</param>
        /// <returns>le tableau de chaînes</returns>
        public static string[] SplitEspaces(string chaine)
        {
            return chaine.Split((char[])null, StringSplitOptions.RemoveEmptyEntries);
        }

        /// <summary>
        /// Sépare une châine de caractères en deux.
        /// </summary>
        /// <param name="chaine">une chaîne</param>
        /// <param name="indice">un indice de séparation</param>
        /// <param name="sousChaine1">une sous-chaîne n°1 (sortie)</param>
        /// <param name="sousChaine2">une sous-chaîne n°2 (sortie)</param>
        /// <param name="inclureIndiceDansSousChaine2">indique s'il faut inclure le caractère d'indice dans la sous-chaîne n°2</param>
        public static void Split(string chaine, int indice, out string sousChaine1, out string sousChaine2, bool inclureIndiceDansSousChaine2 = false)
        {
            sousChaine1 = chaine.Substring(0, indice);
            sousChaine2 = chaine.Substring(indice + (inclureIndiceDansSousChaine2 ? 0 : 1));
        }

        /// <summary>
        /// Sépare une châine de caractères en deux, à partir de la première occurence d'un caractère.
        /// </summary>
        /// <param name="chaine">une chaîne</param>
        /// <param name="caractere">un caractère de séparation</param>
        /// <param name="sousChaine1">une sous-chaîne n°1 (sortie)</param>
        /// <param name="sousChaine2">une sous-chaîne n°2 (sortie)</param>
        /// <param name="inclureIndiceDansSousChaine2">indique s'il faut inclure le caractère d'indice dans la sous-chaîne n°2</param>
        public static void SplitFirstIndex(string chaine, char caractere, out string sousChaine1, out string sousChaine2, bool inclureIndiceDansSousChaine2 = false)
        {
            Split(chaine, chaine.IndexOf(caractere), out sousChaine1, out sousChaine2, inclureIndiceDansSousChaine2);
        }

        /// <summary>
        /// Sépare une châine de caractères en deux, à partir de la dernière occurence d'un caractère.
        /// </summary>
        /// <param name="chaine">une chaîne</param>
        /// <param name="caractere">un caractère de séparation</param>
        /// <param name="sousChaine1">une sous-chaîne n°1 (sortie)</param>
        /// <param name="sousChaine2">une sous-chaîne n°2 (sortie)</param>
        /// <param name="inclureIndiceDansSousChaine2">indique s'il faut inclure le caractère d'indice dans la sous-chaîne n°2</param>
        public static void SplitLastIndex(string chaine, char caractere, out string sousChaine1, out string sousChaine2, bool inclureIndiceDansSousChaine2 = false)
        {
            Split(chaine, chaine.LastIndexOf(caractere), out sousChaine1, out sousChaine2, inclureIndiceDansSousChaine2);
        }

        /// <summary>
        /// Masque une ligne.
        /// </summary>
        /// <param name="tableLayoutPanel">un TableLayoutPanel</param>
        /// <param name="control">un contrôle de la ligne</param>
        public static void HideRow(TableLayoutPanel tableLayoutPanel, Control control)
        {
            int indexLigne = tableLayoutPanel.GetRow(control);
            tableLayoutPanel.RowStyles[indexLigne].SizeType = SizeType.Absolute;
            tableLayoutPanel.RowStyles[indexLigne].Height = 0;
        }

        /// <summary>
        /// Affiche une fenêtre de barre de progression, pour un traitement long.
        /// </summary>
        /// <param name="traitement">le traitement</param>
        /// <param name="isTraitementAnnulable">indique si le traitement est annulable</param>
        /// <param name="nbElementsTraitement">le nombre d'éléments du traitement</param>
        /// <param name="nbEtapesTraitement">le nombre d'étapes du traitement</param>
        /// <param name="titre">le titre de la fenêtre</param>
        /// <returns><code>true</code> si le traitement s'est terminé correctement, <code>false</code> sinon</returns>
        public static bool ShowProgressBarForm(
            Func<BackgroundWorker, bool> traitement,
            bool isTraitementAnnulable,
            int nbElementsTraitement,
            long nbUnitesTraitement,
            int nbEtapesTraitement,
            string titre)
        {
            ProgressBarForm progressBarForm = new ProgressBarForm(
                traitement,
                isTraitementAnnulable,
                nbElementsTraitement,
                nbUnitesTraitement,
                nbEtapesTraitement,
                titre);
            progressBarForm.ShowDialog();
            return progressBarForm.ValeurRetourTraitement;
        }
    }
}
